@@ -76,8 +76,8 @@ func TestFailures(t *testing.T) {
 	defer func() {
 		// to avoid deadlocks during shutdown, we have to not try to send
 		// things out on channels while we're shutting down (this undoes the
-		// logFailure overide about 25 lines down)
-		s.logFailure = func(*Supervisor, Service, string, float64, float64, bool, interface{}, []byte) {}
+		// LogFailure overide about 25 lines down)
+		s.LogFailure = func(*Supervisor, Service, string, float64, float64, bool, interface{}, []byte) {}
 		s.Stop()
 	}()
 	s.sync()
@@ -102,7 +102,7 @@ func TestFailures(t *testing.T) {
 
 	failNotify := make(chan bool)
 	// use this to synchronize on here
-	s.logFailure = func(supervisor *Supervisor, s Service, sn string, cf float64, ft float64, r bool, error interface{}, stacktrace []byte) {
+	s.LogFailure = func(supervisor *Supervisor, s Service, sn string, cf float64, ft float64, r bool, error interface{}, stacktrace []byte) {
 		failNotify <- r
 	}
 
@@ -153,7 +153,7 @@ func TestFailures(t *testing.T) {
 
 	nowFeeder.appendTimes(oneDecayLater)
 	backingoff := make(chan bool)
-	s.logBackoff = func(s *Supervisor, backingOff bool) {
+	s.LogBackoff = func(s *Supervisor, backingOff bool) {
 		backingoff <- backingOff
 	}
 
@@ -276,8 +276,8 @@ func TestDefaultLogging(t *testing.T) {
 
 	name := serviceName(&BarelyService{})
 
-	s.logBadStop(s, service, name)
-	s.logFailure(s, service, name, 1, 1, true, errors.New("test error"), []byte{})
+	s.LogBadStop(s, service, name)
+	s.LogFailure(s, service, name, 1, 1, true, errors.New("test error"), []byte{})
 
 	s.Stop()
 }
@@ -289,8 +289,8 @@ func TestNestedSupervisors(t *testing.T) {
 	super2 := NewSimple("Nested5")
 	service := NewService("Service5")
 
-	super2.logBadStop = func(*Supervisor, Service, string) {
-		panic("Failed to copy logBadStop")
+	super2.LogBadStop = func(*Supervisor, Service, string) {
+		panic("Failed to copy LogBadStop")
 	}
 
 	super1.Add(super2)
@@ -298,7 +298,7 @@ func TestNestedSupervisors(t *testing.T) {
 
 	// test the functions got copied from super1; if this panics, it didn't
 	// get copied
-	super2.logBadStop(super2, service, "Service5")
+	super2.LogBadStop(super2, service, "Service5")
 
 	go super1.Serve()
 	super1.sync()
@@ -356,7 +356,7 @@ func TestStoppingStillWorksWithHungServices(t *testing.T) {
 		return resumeChan
 	}
 	failNotify := make(chan struct{})
-	s.logBadStop = func(supervisor *Supervisor, s Service, name string) {
+	s.LogBadStop = func(supervisor *Supervisor, s Service, name string) {
 		failNotify <- struct{}{}
 	}
 
@@ -380,7 +380,7 @@ func TestRemovingHungService(t *testing.T) {
 	s.getAfterChan = func(d time.Duration) <-chan time.Time {
 		return resumeChan
 	}
-	s.logBadStop = func(supervisor *Supervisor, s Service, name string) {
+	s.LogBadStop = func(supervisor *Supervisor, s Service, name string) {
 		failNotify <- struct{}{}
 	}
 	service := NewService("Service WillHang")
@@ -422,6 +422,10 @@ func TestRemoveService(t *testing.T) {
 	if err != ErrWrongSupervisor {
 		t.Fatal("Did not detect that the ServiceToken was wrong")
 	}
+	err = s.RemoveAndWait(ServiceToken{id.id + (1 << 32)}, time.Second)
+	if err != ErrWrongSupervisor {
+		t.Fatal("Did not detect that the ServiceToken was wrong")
+	}
 }
 
 func TestFailureToConstruct(t *testing.T) {
@@ -485,7 +489,7 @@ func TestFailingSupervisors(t *testing.T) {
 	}
 	failNotify := make(chan string)
 	// use this to synchronize on here
-	s1.logFailure = func(supervisor *Supervisor, s Service, name string, cf float64, ft float64, r bool, error interface{}, stacktrace []byte) {
+	s1.LogFailure = func(supervisor *Supervisor, s Service, name string, cf float64, ft float64, r bool, error interface{}, stacktrace []byte) {
 		failNotify <- fmt.Sprintf("%s", s)
 	}
 
@@ -533,6 +537,78 @@ func TestIssue11(t *testing.T) {
 	subsuper.Add(NewService("may cause data race"))
 }
 
+func TestRemoveAndWait(t *testing.T) {
+	t.Parallel()
+
+	s := NewSimple("main")
+	s.timeout = time.Second
+	s.ServeBackground()
+
+	service := NewService("A1")
+	token := s.Add(service)
+	<-service.started
+
+	// Normal termination case; without the useStopChan flag on the
+	// NewService, this will just terminate. So we can freely use a long
+	// timeout, because it should not trigger.
+	err := s.RemoveAndWait(token, time.Second)
+	if err != nil {
+		t.Fatal("Happy case for RemoveAndWait failed: " + err.Error())
+	}
+	// Removing already-removed service does unblock the channel
+	err = s.RemoveAndWait(token, time.Second)
+	if err != nil {
+		t.Fatal("Removing already-removed service failed: " + err.Error())
+	}
+
+	service = NewService("A2")
+	token = s.Add(service)
+	<-service.started
+	service.take <- Hang
+
+	// Abnormal case; the service is hung until we release it
+	err = s.RemoveAndWait(token, time.Millisecond)
+	if err == nil {
+		t.Fatal("RemoveAndWait unexpectedly returning that everything is fine")
+	}
+	if err != ErrTimeout {
+		// laziness; one of the unhappy results is err == nil, which will
+		// panic here, but, hey, that's a failing test, right?
+		t.Fatal("Unexpected result for RemoveAndWait on frozen service: " +
+			err.Error())
+	}
+
+	// Abnormal case: The service is hung and we get the supervisor
+	// stopping instead.
+	service = NewService("A3")
+	token = s.Add(service)
+	<-service.started
+	s.Stop()
+	err = s.RemoveAndWait(token, 10*time.Millisecond)
+
+	if err != ErrTimeout {
+		t.Fatal("Unexpected result for RemoveAndWait on a stopped service: " + err.Error())
+	}
+}
+
+func TestCoverage(t *testing.T) {
+	New("testing coverage", Spec{
+		LogBadStop: func(*Supervisor, Service, string) {},
+		LogFailure: func(
+			supervisor *Supervisor,
+			service Service,
+			serviceName string,
+			currentFailures float64,
+			failureThreshold float64,
+			restarting bool,
+			error interface{},
+			stacktrace []byte,
+		) {
+		},
+		LogBackoff: func(s *Supervisor, entering bool) {},
+	})
+}
+
 // http://golangtutorials.blogspot.com/2011/10/gotest-unit-testing-and-benchmarking-go.html
 // claims test function are run in the same order as the source file...
 // I'm not sure if this is part of the contract, though. Especially in the
